CREATE EXTENSION pgcrypto;

CREATE EXTENSION pgcryptokey;

-- send shared key to the client
SELECT get_shared_key()
\gset

-- Hash the plain-text password to create the access password,
-- then encrypt it with the shared key
\set enc_access_password `echo 'my secret' | tr -d '\n' | openssl dgst -sha256 -binary | gpg2 --symmetric --batch --cipher-algo AES128 --passphrase :'get_shared_key' | xxd -plain | tr -d '\n'`

-- set access password
SELECT set_session_access_password(:'enc_access_password');

-- create cryptokey and save key_id in psql variable
SELECT create_cryptokey('test', 16)
\gset

-- set and test server variable
SELECT set_cryptokey('test');
SHOW pgcryptokey.key_id;
SHOW pgcryptokey.name;
SELECT length(current_setting('pgcryptokey.key'));

SELECT length(get_cryptokey('test'));

-- access non-existant key
SELECT set_cryptokey(700);

-- use both key access methods
SELECT pgp_sym_decrypt(pgp_sym_encrypt('my test', current_setting('pgcryptokey.key')), get_cryptokey('test'));

-- use cryptokey in a table
CREATE TEMPORARY TABLE cryptokey_sample (data BYTEA);

INSERT INTO cryptokey_sample VALUES (pgp_sym_encrypt('my data', get_cryptokey('test')));

SELECT pgp_sym_decrypt(data, get_cryptokey('test'))
FROM cryptokey_sample;

-- rotate keys and test
SELECT supersede_cryptokey('test', 16);

-- use updated key
SELECT set_cryptokey('test');

-- reencrypt data
UPDATE cryptokey_sample SET data =
	reencrypt_data(data, :create_cryptokey,
		       current_setting('pgcryptokey.key_id')::integer);

-- test reencryption
SELECT pgp_sym_decrypt(data, get_cryptokey('test'))
FROM cryptokey_sample;

-- change the access password of a key
\set enc_access_password2 `echo 'new special secret' | tr -d '\n' | openssl dgst -sha256 -binary | gpg2 --symmetric --batch --cipher-algo AES128 --passphrase :'get_shared_key' | xxd -plain | tr -d '\n'`

SELECT change_key_access_password(current_setting('pgcryptokey.key_id')::integer,
				  :'enc_access_password2');

-- set access password
SELECT set_session_access_password(:'enc_access_password2');

-- test reencryption
SELECT pgp_sym_decrypt(data, get_cryptokey('test'))
FROM cryptokey_sample;

-- must drop superseded key first to avoid notice
SELECT set_session_access_password(:'enc_access_password');
SELECT drop_cryptokey(:create_cryptokey);

SELECT set_session_access_password(:'enc_access_password2');
SELECT drop_cryptokey('test');
